function MyMCC = CONDREG
% CONDREG.M
% Conditions d'application de la rgression linaire
% Guy Mlard, dcembre 1996. gmelard@ulb.ac.be
% MATLAB avec bote  outils statistique
% (DISTCHCK.M, NORMCDF.M, RNDCHECK.M, TCDF.M, TRND.M)

% Traitement automatique ou choix
rep = menu('CONDREG, Guy Mlard, 1996. Traitement','automatique','choix') ;
if rep == 1 
  R = 1000 ;
  T = 10 ;
  H = 3 ;
  beta = 1 ;
else
  R = max(input('Entrez le nombre de simulations '), 1);
  T = input('Entrez le nombre de donnes ') ;
  H = input('Entrez l''horizon de prvision ') ;
  beta = input('Entrez le coefficient de rgression ') ;
end
disp(['Nombre de simulations    : ', int2str(R)]) ;
disp(['Nombre de donnes        : ', int2str(T)]) ;
disp(['Horizon de prvision     : ', int2str(H)]) ;
disp(['Coefficient de rgression: ', num2str(beta)]) ;
% Valeurs par dfaut
if T <= 0, T = 10, end ;
if H <= 0, H = 1, end ;

while 1

C = menu('Entrez la condition non satisfaite', ...
    '(1) aucune', ...
    '(2) variable explicative avec erreurs', ...
    '(3) colinarit', ...
    '(4) relation non linaire', ...
    '(5) non normalit', ...
    '(6) donnes aberrantes', ...
    '(7) htroscdasticit', ...
    '(8) autocorrlation positive des erreurs', ...
    '(9) autocorrlation ngative des erreurs', ...
    '(10) ARRET' ) ;

if C == 10 ; break, end 

% I N I T I A  L I S A T I O N 

% Options 
k = 1              ; % loi normale
pertu  = 2/T       ; % proportion de donnes aberrantes
ampli  = 10        ; % amplitude des perturbations
ervar  = T^0.5     ; % variation des erreurs sur la variable explicative
quadr  = 20/(T^2)  ; % taux de non linarit
heter  = 2/T       ; % pente de la tendance en cart-type
autoc  = min(0.9,2.5/(T^0.5)) ; % autocorrlation des erreurs
if C == 5      
  k = menu('Entrez la loi','normale','uniforme','Laplace', ...
                           'Student  2 d.l.', 'Cauchy') ;
end
X0 = 1:(T + H) ;
Z = zeros(1,T + H) ;
S = ones(1,T + H) ;
Q = zeros(1,T + H) ;
compt1 = zeros(1,1001);
compt2 = zeros(1,1001);
compt3 = zeros(1,1001);

% T R A I T E M E N T

% boucle sur les rptitions
for ir = 1:R
  % on simule les erreurs (et la variable explicative)
  X = X0 ;
  if C == 2, Z = ervar*randn(1, T + H) ; end 
  if C == 3, X = 1 + X0/10000000000 ; end
  if C == 4, Q = quadr*(X0 - T/2).*(X0 - T/2); end 
  if     k == 1 , E = randn(1, T + H) ;
  elseif k == 2 , E = 2*sqrt(3)*(rand(1, T + H) - 0.5) ;
  elseif k == 3 , for i = 1 : (T + H)
                    temp1 = rand(1) ;
                    temp2 = abs( 2*temp1 - 1) ;
                    E(i) =  (1/sqrt(2))*log(1 - temp2) ;
                    if temp1 < 0.5 ; E(i) = - E(i) ; end 
                  end 
  elseif k == 4 , E = trnd(2, 1, T + H) ;
  elseif k == 5 , E = trnd(1, 1, T + H) ;
  end
  if C == 6
    ALEA = rand(1, T + H) ;
    for i = 1 : (T + H)
      if ALEA(i) < pertu , E(i) = E(i) + ampli ; end
    end 
  end 
  if C == 7 , E = E.*(0.5 + heter*X0) ; end ;
  if C == 8 , for i = 2 : (T + H)
                 E(i) =  E(i) + autoc*E(i - 1) ;
              end 
  end
  if C == 9 , for i = 2 : (T + H)
                 E(i) =  E(i) - autoc*E(i - 1) ;
              end 
  end
  % on calcule la relation thorique
  XTH = X + Z ;
  YTH = 1 + beta*XTH + Q ;
  % on calcule les observations de la variable dpendante
  Y = YTH + E ;
  % on prpare le graphique des erreurs (au temps T + h)
  i = min(1000, max(0, round((E(T + H) + 5)*100))) + 1 ;
  compt1(i) = compt1(i) + 1 ;
  % on effectue la rgression linaire simple
  covar = cov([Y(1:T)' X(1:T)']) ;
  xbar = mean(X(1:T)) ;
  ybar = mean(Y(1:T)) ;
  s2x = covar(2,2) ;
  s2y = covar(1,1) ;
  sxy = covar(1,2) ;
  b = sxy/s2x ;
  a = ybar - b*xbar ;
  s2b = (s2y/s2x - b^2)/(T - 2) ;
  s2e = s2y*(1 - (sxy^2)/(s2x*s2y))*(T - 1)/(T - 2) ;
  tstat = (b - beta)/sqrt(s2b) ;
  % on prpare le graphique des statistiques de Student
  i = min(1000, max(0, round((tstat + 5)*100))) + 1 ;
  compt2(i) = compt2(i) + 1 ;
  % on calcule les rsidus et les rsidus studentiss
  YAJ = a + b*X ;
  RES = Y - YAJ ;
  VRES = s2e*(1 + 1/T + ((X(T + H) - xbar)^2)/((T - 1)*s2x)) ;
  NRES = RES(T + H)/(VRES^0.5) ;
  % on prpare le graphique des erreurs (au temps T + h)
  i = min(1000, max(0, round((NRES + 5)*100))) + 1 ;
  compt3(i) = compt3(i) + 1 ;
  % on ralise trois graphiques pour la premire simulation
  if ir ==1
    if C == 1
      Titre = 'Toutes les conditions satisfaites' ;
    elseif C == 2
      Titre = ['Variable explicative avec erreurs d''cart-type ', ...
               num2str(ervar) ] ;
    elseif C == 3
      Titre = 'Colinarit ' ;
    elseif C == 4
      Titre = ['Relation quadratique avec coefficient ', ...
               num2str(quadr) ] ;
    elseif C == 5
      Titre = ['Loi des erreurs numro ', ...
               int2str(k) ] ;
    elseif C == 6
      Titre = ['Donnes aberrantes d''amplitude ', num2str(ampli), ...
               ', probabilit ', num2str(pertu) ] ;
    elseif C == 7
      Titre = ['Htroscdasticit avec cart-type de 0,5  ', ...
               num2str(heter*T) ] ;
    elseif C == 8
      Titre = ['Autocorrlation positive des erreurs de ', ...
               num2str(autoc) ] ;
    elseif C == 9
      Titre = ['Autocorrlation ngative des erreurs de ', ...
               num2str(autoc) ] ;
    end
    % on ralise le graphique avec les donnes, la rgression thorique,
    % et la rgression estime
    disp(Titre) ;
    plot(X(1:T), Y(1:T), 'ro')
    hold on
    plot(X(1:T + H), YAJ(1:T + H), 'r--')
    hold on
    plot(XTH(1:T + H), YTH(1:T + H), 'g-')
    hold off
    title([Titre, '. Donnes d''un exemple'])
    xlabel('X')
    ylabel('Y obs.(o), thor.(-), aj.(- -)')
    pause
    % on ralise le graphique des rsidus en fonction des valeurs ajustes
    LIMP = +1.96*ones(1, T) ;
    LIMM = -1.96*ones(1, T) ;
    plot(YAJ(1:T), RES(1:T), 'rx')
    hold on
    plot(YAJ(1:T), LIMP, 'g:') ;
    hold on
    plot(YAJ(1:T), LIMM, 'g:') ;    
    hold off
    title([Titre, '. Analyse des rsidus'])
    xlabel('valeur ajuste')
    ylabel('rsidu')
    pause
    % on ralise le graphique des rsidus en fonction du temps
    plot(X0(1:T), RES(1:T), 'r-')
    hold on
    plot(X0(1:T), LIMP, 'g:') ;
    hold on
    plot(X0(1:T), LIMM, 'g:') ;    
    hold off
    title([Titre, '. Analyse des rsidus'])
    xlabel('temps')
    ylabel('rsidu')
    pause
  end
end

% C L O T U R E

for i = 2:1001
  compt1(i) = compt1(i - 1) + compt1(i) ;
  compt2(i) = compt2(i - 1) + compt2(i) ;
  compt3(i) = compt3(i - 1) + compt3(i) ;
end
nv = ((1:1001) - 501)/100 ;
fdn = normscdf(nv) ;
compt1 = compt1/R ; 
plot(nv, compt1, 'r:')
hold on
plot(nv, fdn,'g-')
hold off
title([Titre, '. Distribution des erreurs'])
xlabel(['Erreur du temps ', int2str(T + H)])
ylabel('probabilit(-), frquence(- -)')
pause
tv = ((1:1001) - 501)/100 ;
fdt = tcdf(tv, T - 2) ;
compt2 = compt2/R ; 
plot(tv, compt2, 'r:')
hold on
plot(tv, fdt,'g-')
hold off
title([Titre, '. Statistique t de Student'])
xlabel('statistique t')
ylabel('probabilit(-), frquence(- -)')
pause
nv = ((1:1001) - 501)/100 ;
fdn = normscdf(nv) ;
compt3 = compt3/R ; 
plot(nv, compt3, 'r:')
hold on
plot(nv, fdn,'g-')
hold off
title([Titre, '. Distribution des rsidus'])
xlabel(['Rsidu du temps ', int2str(T + H)])
ylabel('probabilit(-), frquence(- -)')
pause

end